import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

class AbstractController implements WindowListener {
  protected JFrame view;
  protected Component opener;
  
  public AbstractController(){
    this(null);
  }
  
  public AbstractController(Component opener){
    this.opener = opener;
  }
  
  public void show(){
    if (opener != null){
      opener.setEnabled(false);
    }
    
    DialogWorker dw = new DialogWorker(){
      public Boolean doInBackground(){
        init();
        return Boolean.TRUE;
      }
      
      public void doneBeforeDispose() throws Exception {
        getAll();
        initGUI();
      }
      
      public void doneAfterDispose(){
        try {
          getAll();
          if (opener != null){
            opener.setVisible(false);
          }
          if (view != null){
            view.setVisible(true);
          }
        } catch (Exception e){
          JOptionPane.showMessageDialog(opener,e);
          windowClosing(null);
        }
      }
    };
    dw.executeWithDialog((Frame)opener, "N");
  }
  
  protected void init(){}
  protected void initGUI(){}
  public void windowActivated(WindowEvent e) {}
  public void windowClosed(WindowEvent e) {}
  public void windowClosing(WindowEvent e) {
    if (opener != null && opener.isDisplayable()){
      opener.setVisible(true);
      opener.setEnabled(true);
    }
    if (e.getSource() instanceof Window){
      ((Window)e.getSource()).dispose();
    }
  }
  public void windowDeactivated(WindowEvent e) {}
  public void windowDeiconified(WindowEvent e) {}
  public void windowIconified(WindowEvent e) {}
  public void windowOpened(WindowEvent e) {}
}
